library(tidyverse)
library(scales)


data <- readr::read_delim('data.tsv', delim='\t')

probs <- problems(data)

if (!is.null(probs) && nrow(probs) > 0) {
  stopifnot(all(probs$expected=='6 columns' &
                probs$actual=='1 columns'))
}

GetQuantile <- function(vec, q) {
  sorted <- sort(vec)
  index <- ceiling(length(vec)*q)
  sorted[[index]]
}

GetHigh <- function(vec) {
  sorted <- sort(vec)
  len <- length(vec)
  index <- floor(len*.999)
  sorted[[index]]
}

Bucket <- function(vec, box_length=10) {
  max_box <- max(vec)/box_length + 1
  cuts <- c(0, seq(max_box)*box_length)
  labels <- sprintf('%s-%s', cuts-box_length, cuts)
  labels[[1]] <- "0 Exactly"
  cuts <- c(-1, cuts)
  cut(vec, cuts, labels=labels, right=TRUE)

}

RemoveHigh <- function(vec) {
  sorted <- sort(vec)
  len <- length(vec)
  index <- floor(len*.999)
  box_length <- 10
  cuts <- 1:200*box_length
  labels <- sprintf('%s-%s', cuts-box_length, cuts)
  cuts <- c(0, cuts)
  print(length(cuts))
  print(length(labels))
  remove_top <- head(sorted, index)
  cut(remove_top, cuts, labels=labels)
}

CleanNumber <- function(num) {
  (num
    |> str_replace_all(',', '')
    |> str_replace_all('Days: ', '')
    |> as.numeric())
}

clean <- (data
  |> filter(!is.na(url))
  |> group_by(url)
  |> filter(row_number()==n())
  |> ungroup()
  |> mutate(anime_time=CleanNumber(anime_time),
            manga_time=CleanNumber(manga_time),
            bucket_anime_time=Bucket(anime_time),
            bucket_manga_time=Bucket(manga_time))
  |> filter(!(anime_time==0 &
            manga_time==0) &
            anime_time <= GetHigh(anime_time) &
            manga_time <= GetHigh(manga_time)))

summary(select(clean, anime_time, manga_time))
(clean
  |> filter(anime_time > 0)
  |> select(anime_time)
  |> summary())

(clean
  |> filter(manga_time > 0)
  |> select(manga_time)
  |> summary())

EveryN = function(n) {
  return(function(x) {x[c(TRUE, rep(FALSE, n - 1))]})
}

UnivariatePropPlot <- function(df, col, by=.05) {
  total_rows <- nrow(df)
  prop_df <- (df
    |> group_by_at(col)
    |> summarize(prop=n()/total_rows, n=n(), .groups="drop"))
  all_labels <- sort(unique(df[[col]]))
  use_breaks <- c(TRUE, rep(c(TRUE, rep(FALSE, 4)), length(all_labels)))
  numeric_breaks <- all_labels[use_breaks]

  ggplot(prop_df, aes(x=get(col), y=prop)) +
    geom_col(fill='dark blue', color='white') +
    ylab("Percent of usres") +
    xlab("Time (days)") +
    scale_x_discrete(breaks=EveryN(4), expand=c(0, 1)) +
    scale_y_continuous(limits = c(0, NA), expand = c(0,0), labels=scales::percent, breaks=seq(0,1,by=by)) +
    theme(panel.background=element_blank(), panel.border=element_blank(),
    panel.grid.major=element_blank(),
    panel.grid.major.y=element_line(color='grey60', linetype='dashed'),
    axis.text=element_text(color='black', margin=-100),
    axis.text.x=element_text(angle=90, vjust=.3),
    plot.title=element_text(color='black', size=16))
}



anime_plot <- UnivariatePropPlot(clean, 'bucket_anime_time', by=.01) +
  ggtitle("Distribution of time spent watching anime")

manga_plot <- UnivariatePropPlot(clean, 'bucket_manga_time') +
  ggtitle("Distribution of time spent reading manga")

DecilePlot <- function(df, x_col, y_col, buckets=10) {
  error <- qnorm(.975)
  (df
    |> mutate(decile=ntile(get(x_col), buckets))
    |> group_by(decile)
    |> summarize(val=mean(get(y_col)), s=sd(get(y_col))/sqrt(n()), .groups="drop")
    |> mutate(decile=factor(decile, levels=sort(unique(decile)), labels=as.character(sort(unique(decile)))))
    |> ggplot(aes(x=decile, y=val)) +
  geom_point() +
  geom_errorbar(aes(ymin=val-s*error, ymax=val+s*error)) +
  theme(panel.background=element_blank(), panel.border=element_blank(),
    panel.grid.major=element_line(color='grey60', linetype='dashed'),
    axis.text=element_text(color='black', margin=-100), plot.title=element_text(color='black', size=16)))
}

BucketAveragePlot <- function(df, x_col, y_col) {
  error <- qnorm(.975)
  (df
    |> group_by(bucket=get(x_col))
    |> summarize(val=mean(get(y_col)), s=sd(get(y_col))/sqrt(n()), .groups="drop")
    |> ggplot(aes(x=bucket, y=val)) +
  geom_point() +
  geom_errorbar(aes(ymin=val-s*error, ymax=val+s*error)) +
  scale_x_discrete(breaks=EveryN(n=3), expand=c(0, 1)) +
  theme(panel.background=element_blank(), panel.border=element_blank(),
    panel.grid.major=element_line(color='grey60', linetype='dashed'),
    axis.text=element_text(color='black', margin=-100), plot.title=element_text(color='black', size=16)))

}

anime_vs_manga_lm <- lm(clean$manga_time ~ clean$anime_time)
summary(anime_vs_manga_lm)

anime_vs_manga_plot_log <- ggplot(clean, aes(x=anime_time, y=manga_time)) +
  scale_x_log10(labels=comma) +
  scale_y_log10(labels=comma) +
  geom_point()


anime_vs_manga_plot <- ggplot(filter(clean, anime_time < 400), aes(x=anime_time, y=manga_time)) +
  geom_point()

time_vs_score <- ggplot(filter(clean, anime_score > 0), aes(x=anime_time, y=anime_score)) +
  geom_point()

DecilePlot(clean, x_col='anime_time', y_col='anime_score', buckets=20)
DecilePlot(clean, x_col='manga_time', y_col='manga_score')
DecilePlot(clean, x_col='anime_time', y_col='manga_score')

anime_vs_manga_percentile <- DecilePlot(clean, x_col='anime_time', y_col='manga_time', buckets=100) +
  scale_x_discrete(breaks=EveryN(10), expand=c(0, 1)) +
  xlab('Percentile') +
  ylab('Average manga read time') +
  ggtitle('Manga read time vs anime watch time percentile')


DecilePlot(clean, x_col='anime_time', y_col='manga_time', buckets=10)

anime_vs_anime_score <- BucketAveragePlot(filter(clean, anime_time<300 & anime_time > 0), x_col='bucket_anime_time', y='anime_score') +
  ylab("Average anime score") +
  xlab("Total anime watch time (Days)")

anime_vs_anime_score_percent <- DecilePlot(filter(clean, anime_time > 0), x_col='bucket_anime_time', y_col='anime_score', 100) +
  ggtitle("Watch time percentile vs score") +
  scale_x_discrete(breaks=EveryN(5)) +
  ylab("Average anime score") +
  xlab("Watch time percentile")

manga_vs_manga_score <- BucketAveragePlot(filter(clean, manga_time<300 & manga_time > 0), x_col='bucket_manga_time', y='manga_score') +
  ylab("Average manga score") +
  xlab("Total manga read time (Days)")

anime_vs_manga_bucket <- BucketAveragePlot(filter(clean, anime_time<300), x_col='bucket_anime_time', y='manga_time') +
  ylab("Average manga read time") +
  xlab("Anime watch time") +
  ggtitle("Manga read time vs Anime watch time (credible groups)")

BucketAveragePlot(clean, x_col='bucket_anime_time', y='manga_time')

BucketAveragePlot(clean, x_col='bucket_anime_time', y='anime_score')

favorite_anime_data <- readr::read_delim('data_top_10_anime.tsv', delim='\t', show_col_types=FALSE)

shounen_anime <- c('Naruto: Shippuuden', 'Bleach', 'Boku no Hero Academia', 'One Piece', 'Death Note', 'Hunter x Hunter (2011)')
maho_shojo <- c('Cardcaptor Sakura', 'Bishoujo Senshi Sailor Moon', 'Mahou Shoujo Madoka★Magica', 'Shoujo Kakumei Utena', 'Yuuki Yuuna wa Yuusha de Aru', 'Futari wa Precure')
gainax <- c('Shinseiki Evangelion', 'FLCL', 'Tengen Toppa Gurren Lagann', 'Ouritsu Uchuugun: Honneamise no Tsubasa', 'Top wo Nerae! Gunbuster', 'Kareshi Kanojo no Jijou')
top_anime <- c('Sousou no Frieren', 'Fullmetal Alchemist: Brotherhood', 'Steins;Gate', 'Gintama', 'Shingeki no Kyojin', 'Ginga Eiyuu Densetsu')
my_anime <- c('Yu☆Gi☆Oh! Duel Monsters', 'Fate/Zero', 'Bakemonogatari', 'Kizumonogatari III: Reiketsu-hen', 'Trigun', 'Little Witch Academia (TV)')
sam_anime <- c('Mirai Nikki (TV)', 'Yu☆Gi☆Oh! Duel Monsters', 'Chainsaw Man', 'Mushoku Tensei: Isekai Ittara Honki Dasu', 'Re:Zero kara Hajimeru Isekai Seikatsu', 'ReLIFE')

Likes <- function(df, sample_anime) {
  good <- FALSE
  for (col in seq(0, 9)) {
    col <- as.character(col)
    good <- good | if_else(is.na(df[[col]]), FALSE, df[[col]] %in% sample_anime)
  }
  good
}

clean_favorite <- (favorite_anime_data
  |> mutate(shounen=Likes(favorite_anime_data, shounen_anime),
            maho_shojo=Likes(favorite_anime_data, maho_shojo),
            gainax=Likes(favorite_anime_data, gainax),
            top_anime=Likes(favorite_anime_data, top_anime),
            my_anime=Likes(favorite_anime_data, my_anime))
  |> _[c('user', 'shounen', 'maho_shojo', 'gainax', 'top_anime', 'my_anime')])

tidy_favorite_join <- (clean_favorite
  |> inner_join(clean, by=c(user='username'))
  |> filter(anime_time > 0)
  |> mutate(bucket=ntile(anime_time, 100))
  |> group_by(bucket)
  |> summarize(Shounen=mean(shounen),
               `Magical Girl`=mean(maho_shojo),
               Gainax=mean(gainax),
               `Top Rated`=mean(top_anime),
               `Anime I like`=mean(my_anime),
               .groups="drop")
  |> pivot_longer(cols=all_of(c('Shounen', 'Magical Girl', 'Gainax', 'Top Rated', 'Anime I like'))))

taste_plot <- ggplot(data=tidy_favorite_join, aes(x=bucket, y=value, color=name, group=name)) +
  geom_point() +
  geom_line() +
  xlab('Watch time percentile') +
  ylab('Probability of anime appearing in top 10') +
  ggtitle("Popularity of different kinds of anime") +
  scale_y_continuous(labels=scales::percent, breaks=seq(0,1,by=.05)) +
  labs(color="Type of anime") +
  theme(panel.background=element_blank(), panel.border=element_blank(),
    panel.grid.major=element_line(color='grey60', linetype='dashed'),
    axis.text=element_text(color='black', margin=-100), plot.title=element_text(color='black', size=16))



ggsave('anime_plot.png', anime_plot, height=5)
ggsave('manga_plot.png', manga_plot, height=5)
ggsave('anime_vs_manga_bucket.png', anime_vs_manga_bucket, height=5)
ggsave('taste_plot.png', taste_plot, height=5)
ggsave('anime_vs_manga_percentile.png', anime_vs_manga_percentile, height=5)
ggsave('anime_vs_anime_score.png', anime_vs_anime_score, height=5)
ggsave('manga_vs_manga_score.png', manga_vs_manga_score, height=5)

print(':)')

# plt <- (ggplot(clean,aes(x=anime_time))
#   +geom_histogram(aes(y=(..count..)/sum(..count..)), binwidth=.1, color='white', fill='dark blue')
#   +scale_x_log10(labels=comma, breaks=c(0,1,3,10,30,100,300,1000))
#   +scale_y_continuous(labels=percent)
#   +xlab("Time (days)")
#   +ylab("Amount")
#   +ggtitle("Distrabution of time spent watching anime on MAL")
#   +theme(panel.background=element_blank(), panel.border=element_blank(),
#   panel.grid.major=element_blank(),
#   panel.grid.major.y=element_line(color='grey60', linetype='dashed'),
#   axis.text=element_text(color='black'), plot.title=element_text(color='black', size=16))
#   )
